<?php
/**
 
 * Internal linking
 * 
 * @copyright (c) 2022 Afrorus ( afrorus.tk )
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 * @author Afrorus (afrorus.tk)
 */

namespace afrorus\intlink\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event listener
 */
class listener implements EventSubscriberInterface
{

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\language\language */
	protected $language;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;




	/**
	 * Constructor of event listener
	 *
  	 * @param \phpbb\language\language             $language          Language object
	 * @param \phpbb\config\config			$config	Config object
	 * @param \phpbb\user   		       $user	          User object
	 * @param \phpbb\auth\auth                                 $auth
	 * @param \phpbb\db\driver\driver_interface		$db					DBAL object
	 */
	public function __construct(\phpbb\language\language $language, \phpbb\config\config $config, \phpbb\user $user, \phpbb\auth\auth $auth, \phpbb\db\driver\driver_interface $db)
	{
		$this->language = $language;
		$this->config = $config;
		$this->user = $user;        
		$this->auth = $auth;        
		$this->db = $db;
	}

	/**
	 * Assign functions defined in this class to event listeners in the core
	 *
	 * @return array
	 */
	static public function getSubscribedEvents()
	{
		return array(
			'core.message_parser_check_message'			=> 'auto_internal_link',
		);
	}

	# возвращает массив с id материнского и  дочерних разделов
	public function near($forum_id)
	{

		global  $db;
		$sql = 'SELECT forum_id FROM ' . FORUMS_TABLE . ' WHERE (parent_id = '.$forum_id. ' OR forum_id = '.$forum_id.' )';
		$result = $db->sql_query($sql, 600);
			
		$ii = 0;
		$rowset = array();
		# перебираем выборку и записываем построчно в массив
		while ($row = $db->sql_fetchrow($result))
			{
				$rowset[$ii] = $row;
				$rowset[$ii] = $rowset[$ii]['forum_id'];
				$ii++;		
			}

		$db->sql_freeresult($result);

		# смотрим какой форум для текущего материнский
		$sql = 'SELECT parent_id AS forum_id FROM ' . FORUMS_TABLE . ' WHERE (forum_id = '.$forum_id. ' )';
		$result = $db->sql_query($sql, 600);

		# добавляем его в массив
		while ($row = $db->sql_fetchrow($result))
			{
				$rowset[$ii] = $row;
				$rowset[$ii] = $rowset[$ii]['forum_id'];
				$ii++;		
			}


		# очищаем память
		$db->sql_freeresult($result);

		return $rowset;
	}



	/**
	 * Assigns template vars
	 * 
	 * @param object $event The event object
	 * @return void
	 */
	public function auto_internal_link($event)
	{
		global $request;
		# что указано в конфиге: 0 - всё отключено, 1 - форумы, 2 - топики, 3 - и топики и форумы
		$options = $this->config['afrorus_intlink'];
		# минимальная длина темы\раздела, которая учитывается расширением
		$len = $this->config['afrorus_intlink_len'];
		# учитывать темы только в текущем разделе+подразделы
		$only_this = $this->config['afrorus_intlink_only_this'];
	
		# код начала и конца записи номера ячейки, случайные 10 символов
		$kod_nachala = bin2hex(random_bytes(5));
		$kod_konca =  bin2hex(random_bytes(5));
		# код конца названия темы в тексте
		$kod_konca_temy = bin2hex(random_bytes(5));
		
		# если отключены и темы, и разделы, то выходим - делить ничего не надо		
		if ($options == 0)
			{
				return;	
			}
		# надо выяснить -  новый это пост или редактирование старого, способ кривой, но как по другому сделать не знаю ((
		$mmode = $request->server('HTTP_REFERER');
		# само сообщение, которое будем обрабатывать
		$msg = $event['message'];
		# пробелы чтобы слова в начале и конце сообщения тоже учитывались, ведь только с пробелами перв и после фразы будет произведена замена, иначе получится что если есть тема "рог",
		# то в слове "пирог" тоже сработает замена, и получится ерунда. В конце удалим эти пробелы из сообщения
		$msg = " ".$msg." ";

		# делаем копию нашего отправляемого сообщения, но уберем оттуда тэг URL целиком, а QUOTE лишь в части ссылки на автора, оставив сам текст для возможной ссылки
		$msgw = $msg;

		$xquote =  strpos($msgw, "[quote=");
		while ($xquote <> false)
			{
			        $xend = strpos($msgw, "]", $xquote);
				$msgw = substr_replace($msgw, " ", $xquote, $xend - $xquote+1);
				$xquote =  strpos($msgw, "[quote=");
  			
			}
				$msgw = str_replace("[/quote]", " ", $msgw);


		$xquote =  strpos($msgw, "[url=");
		while ($xquote <> false)
			{
			        $xend = strpos($msgw, "[/url]", $xquote);
				$msgw = substr_replace($msgw, " ", $xquote, $xend - $xquote+6);
				$xquote =  strpos($msgw, "[url=");
  			
			}



		# тескт поста в нижнем регистре
		$msgl = mb_strtolower($msgw);
        	# проверка новый ли пост или это редактирование старого
		if (strpos($mmode, 'posting.php?mode=edit') == false)

			{
				global  $db;
			        $g1 = $this->auth->acl_getf('f_read', true); # проверяем какие форумы доступны пользователю
				$g2 = array_unique(array_keys($g1)); # записываем их список в массив, это будем использовать для вывода ссылок на  разделы форума 
								     # а для тем используем массив $g3, т.к. тут еще возможно нужна будет фильтрация по текущему разделу и его подразделам
				
				$g3 = $g2;
                                # номер текущей темы, будет 0, если это создание новой темы
				$topic_id = $request->variable('t', 0);

				if ($only_this == 1)
					{
						# текущий раздел форума
						$forum_id = $request->variable('f', 0);

						#  если сообщение создаётся через Быстрый ответ, то форум почему-то иногда не определяется
						# в этом случае делаем запрос к БД, чтобы выяснить к какому разделу относится тема
						if ($forum_id == 0)
							{
								# смотрим какой раздел у этой темы
								$sql = 'SELECT forum_id AS forum_id FROM ' . TOPICS_TABLE . ' WHERE (topic_id = '.$topic_id. ' )';
								$result = $db->sql_query($sql, 600);
								while ($row = $db->sql_fetchrow($result))
									{
										$rowset[0] = $row;
										$forum_id = $rowset[0]['forum_id'];
									}
									# очищаем память
									$db->sql_freeresult($result);
							
							}
						# получаем массив, в котором текущий раздел и его подразделы
						$good_forum = $this->near($forum_id);
						# сюда записываем пересечение с разрешенными текущему пользователю  разделами
						$g3 = array_uintersect($good_forum, $g2,  "strcasecmp");




					
					}
						# выбираем темы, упорядочиваем по длине топика, чтобы сначала делать длинные ссылки, например  на фразу "Жилье в Москве", а не на "жильё"
			                        # если есть несколько тем с одинаковым названием, то наверху окажется более свежая тема, чтобы и на них появлялись ссылки
						# есть тема Жильё в Москве, в тексте написано ЖИЛЬЁ В МОСКВЕ нужна замена ЖИЛЬЁ В МОСКВЕ на ЖИЛЬЁ В МОСКВЕ с ссылкой
						# т.е. в списке тем должны быть хотя бы такие варианты Жильё  в Москве и ЖИЛЬЁ В МОСКВЕ и жильё в москве
								# готовим строку со списком доступных пользователю разделов форума, чтобы вставить в SQL-запрос
								$ii = 0;
								while ($ii < count($g3))
									{
										$forums = $forums.$g3[$ii].",";
										$ii++;
									}
								
								if ($options == 2 OR $options == 3)							
									{
										# в том числе исключаем текущую тему из наших результов, т.к. нет смысла ставить ссылку на саму себя
										$sql = 'SELECT MAX(topic_id) as topic_id, topic_title, forum_id, 1 as type FROM ' . TOPICS_TABLE . ' WHERE (CHAR_LENGTH(topic_title) >= '.$len.'  AND topic_id <> '.$topic_id.' AND FIND_IN_SET(forum_id, "'.$forums.'")) GROUP BY LOWER(topic_title) ORDER BY LENGTH(topic_title) DESC';

                                                                                $result = $db->sql_query($sql, 600);
									}
	
        	                                                if ($options == 1 OR $options == 3)
									{
								        	# получаем общий список форумов
										$sql = 'SELECT forum_id AS topic_id, forum_name AS topic_title, 2 as type FROM ' . FORUMS_TABLE . ' WHERE (CHAR_LENGTH(forum_name) >= '.$len.' AND FIND_IN_SET(forum_id, "'.$forums.'")) GROUP BY forum_name ORDER BY LENGTH(forum_name) DESC';
										$result_forums = $db->sql_query($sql, 600);
									}
			
		 	
				                                # в этот массив запишем результаты нашего запроса к БД
       								$rowset = array();
								$ii = 0;
								# а в этот - те же названия тем, но в нижнем регистре, это понадобится для поиска фраз без учёта регистра
       								$rowsetl = array();

								# перебираем выборку и записываем построчно в массив, сначала темы
								while ($row = $db->sql_fetchrow($result))
									{
										$rowset[$ii] = $row;
										$rowsetl[$ii] = $row;
										$rowsetl[$ii]['topic_title'] = mb_strtolower($rowset[$ii]['topic_title']);
										$ii++;	
	
										# упрощенный вариант названия темы - без знаков препинания и всяких символок\смайликов, заменяем их пробелами							

										$rowset[$ii] = $row;
										$rowsetl[$ii] = $row;
										$rowsetl[$ii]['topic_title'] = trim(mb_strtolower(str_replace(['?','!','.',',',')','(',':',';','*','#','%'], [' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '], $rowsetl[$ii]['topic_title'])));
										# если знаки препинания между словами, то вместо них появятся пробелы, убираем двойные
								                $rowsetl[$ii]['topic_title'] = preg_replace('/[\s]{2,}/', ' ', $rowsetl[$ii]['topic_title']);
										# может быть сиутация, что нахвание темы состоит сплошь из знаков препинания, тогда после упрощения название будет состоять из пробелов
										# в этом случае не переводим счетчик вперед, и в массив на всякий случай записываем оригинал темы
                                                                		if (strlen(trim($rowsetl[$ii]['topic_title'])) > 0)
											{	
												$ii++;		
											}
										else
											{
 												$rowsetl[$ii]	 = $rowset[$ii-1];
											}
						

									}
								# потом названия форумов
								while ($row = $db->sql_fetchrow($result_forums))
									{
										$rowset[$ii] = $row;
										$rowsetl[$ii] = $row;
										$rowsetl[$ii]['topic_title'] = mb_strtolower($rowset[$ii]['topic_title']);													
										$ii++;		
									}


							        # очищаем память
					       			$db->sql_freeresult($result);
					       			$db->sql_freeresult($result_forums);


								$ii = 0;
								$flag = 0;
								# перебираем массив с темами форумами и создаем два новых массива для  замены  слов (фраз) в тексте сообщения на наши с метками
								while ($ii < count($rowset) and $flag == 0)
									{
												# в этом массиве - темы в нижнем регистре
												#$my_topic_name_lower[$ii] = " ".mb_strtolower($rowset[$ii]['topic_title'])." ";
												$my_topic_name_lower[$ii] = " ".$rowsetl[$ii]['topic_title']." ";
												
												# здесь названия тем с нашими метками, они будут нужны при разборе текста , чтобы сохранить 
												# оригинальный регистр написания фраз в посте
												$my_topic_id_l[$ii] = $kod_nachala.$ii.$kod_konca.$my_topic_name_lower[$ii].$kod_konca_temy;

												# темы у нас упорядочены от самый длинных к самым коротким, перебираем их сверху
												# и при первом же срабатывании замены в тексте ($flag) останавливаем перебор
												# т.о. у нас получается в одном сообщении лишь одна ссылка на тему форума - на самую длинную тему
												# но если включена и обработка разделов форума, то ссылок может быть больше за счет этого
												# этог- ограничение сложно убрать, т.к. могут быть темы "желтые одуванчики москвы и области" и "одуванчики москвы"
												# при используемом алгоритме замены (str_replace) такой текст заменится на ссылки два раза, и будет чехарда
												$msgl = str_replace($my_topic_name_lower[$ii], $my_topic_id_l[$ii], $msgl, $flag);                                                                       									
										$ii++;
									} 

								$ii = 0;
								# смещение в тексте, приведенном к нижнем регистру
								$off_l = 0;
								# смещение в оригинальном тексте
								$off_o = 0;
								# сколько замен сделали
								$kolich = substr_count($msgl, $kod_nachala);						
								# массив для окончательных URL - фраза в оригинале со ссылкой на тему в любом регистре
								$tema_org = array();
								# массив для оригинальных фраз - которые будем заменять на ссылки
								$name_org = array();
								if ($kolich > 0)
									{
										while ($ii < $kolich)
											{
											    # начало нашей ячейки
		    									    $xx = mb_strpos($msgl, $kod_nachala, $off_l, 'UTF-8')+iconv_strlen($kod_nachala, 'UTF-8');
											    # конец ячейки
											    $yy = mb_strpos($msgl, $kod_konca, $off_l, 'UTF-8');
											    # конец названия темы
											    $zz = mb_strpos($msgl, $kod_konca_temy, $off_l, 'UTF-8');

											    $nomer = mb_substr($msgl, $xx, $yy-$xx, 'UTF-8');
											    $off_l = $zz+iconv_strlen($kod_konca_temy, 'UTF-8');
											    # тема в нижнем регистре
											    $tema = mb_substr($msgl, $yy+iconv_strlen($kod_konca, 'UTF-8'), $zz-$yy-iconv_strlen($kod_konca, 'UTF-8'), 'UTF-8');
	
											    # тема, как она написана в оригинальном размещаемом посте
											    $tema_orig[$ii] =  mb_substr($msgw, mb_stripos($msgw, $tema, $off_o, 'UTF-8'), iconv_strlen($tema, 'UTF-8'), 'UTF-8');

											    # если это тема, то делаем соответствующий URL
											    if ($rowset[$nomer]['type'] == 1)
												{
												    $url_orig[$ii] =  ' [url=viewtopic.php?t='.$rowset[$nomer]['topic_id'].']'.trim($tema_orig[$ii]).'[/url] ';	
												} 
											   # если раздел форума
											    elseif ($rowset[$nomer]['type'] == 2)
												{
												    $url_orig[$ii] =  ' [url=viewforum.php?f='.$rowset[$nomer]['topic_id'].']'.trim($tema_orig[$ii]).'[/url] ';	
												}

                                                                                            # высчитываем смещение
											    $off_o = mb_strpos($msgw, $tema_orig[$ii], $off_o, 'UTF-8')+iconv_strlen($tema_orig[$ii], 'UTF-8');

											    $ii++;
											}
									}
								# делаем замену в исходном тексте
								$msg = str_replace($tema_orig, $url_orig, $msg);
		        				
                                                                                                 

			}

        $msg = trim($msg);
	$event['message'] = $msg;
		
	}

}
